@@ -2,6 +2,8 @@ use crate::ast::quant::write_quant_format;
 use crate::ast::{ProtoModel, QuantFormat};
 use crate::internal::*;
 use std::io::Read;
+#[cfg(target_family="unix")]
+use std::os::unix::prelude::OsStrExt;
 use std::path::Path;
 
 pub fn stdlib() -> Vec<FragmentDef> {
@@ -195,6 +197,11 @@ fn read_stream<R: std::io::Read>(
     tensors: &mut Vec<(String, Arc<Tensor>)>,
     quantization: &mut Option<HashMap<String, QuantFormat>>,
 ) -> TractResult<()> {
+    // ignore path with any component starting with "." (because OSX's tar is weird)
+    #[cfg(target_family="unix")]
+    if path.components().any(|name| name.as_os_str().as_bytes().get(0) == Some(&b'.')) {
+        return Ok(())
+    }
     if path.file_name().map(|n| n == "graph.nnef").unwrap_or(false) {
         let mut t = String::new();
         reader.read_to_string(&mut t)?;
